@@ -306,7 +306,7 @@ class TestStylesheetCooking(CSSRegistryTestCase.CSSRegistryTestCase):
         self.failUnless('background-color' in all)
         self.failUnless('<link' in all)
         self.failUnless('/%s' % magic_ids[1] in all)
-        self.failIf('/test_rr_1.css' in all)
+        self.failIf('/test_rr_1.css"' in all)
 
     def testReenderingConcatenatesInline(self):
         self.tool.registerStylesheet('test_rr_1.css', rendering='inline')
@@ -816,6 +816,13 @@ class TestResourcePermissions(CSSRegistryTestCase.CSSRegistryTestCase):
         except Unauthorized:
             self.fail()
 
+    def testAuthorizedOnPublish(self):
+        # FIXME - As a manager this should be accessible, but the test doesn't work
+        # when tested by hand in the browser, it does work as expected
+        self.setRoles(['Manager'])
+        response = self.publish(self.toolpath + '/testroot.css')
+        self.failUnlessEqual(response.getStatus(), 200)
+
 class TestDebugMode(CSSRegistryTestCase.CSSRegistryTestCase):
 
     def afterSetUp(self):
@@ -979,6 +986,178 @@ class TestSkinAwareness(CSSRegistryTestCase.CSSRegistryTestCase):
         self.assertEqual(response.getStatus(), 200)
         self.failUnless('purple' in str(response))
 
+
+class TestCSSCompression(CSSRegistryTestCase.CSSRegistryTestCase):
+    def afterSetUp(self):
+        self.tool = getattr(self.portal, CSSTOOLNAME)
+
+    def testCommentCompression(self):
+        input = """
+        /* this is a comment */
+        #testElement {
+            property: value; /* another comment */
+        }
+        /**********/
+        /* this is a multi
+           line comment */
+        #testElement {
+            /* yet another comment */
+            property: value;
+        }
+        """
+        expected = """/* */
+#testElement {
+property: value; /* */
+}
+/* */
+#testElement {
+/* */
+property: value;
+}
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+    def testNewlineCompression(self):
+        input = """
+        
+        
+        /* this is a comment */
+        
+        #testElement {
+            property: value; /* another comment */
+        }
+        
+        /* this is a multi
+           line comment */
+        #testElement {
+        
+            /* yet another comment */
+            property: value;
+            
+        }
+        
+        
+        """
+        expected = """/* */
+#testElement {
+property: value; /* */
+}
+/* */
+#testElement {
+/* */
+property: value;
+}
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+    def testCommentHacks(self):
+        # see http://www.dithered.com/css_filters/index.html
+        input = """
+        #testElement {
+            property/**/: value;
+            property/* */: value;
+            property /**/: value;
+            property: /**/value;
+        }
+        """
+        expected = """#testElement {
+property/**/: value;
+property/* */: value;
+property /**/: value;
+property: /**/value;
+}
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        selector/* */ {  }
+        """
+        expected = """selector/* */ {  }
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        selector/* foobar */ {  }
+        """
+        expected = """selector/* */ {  }
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        selector/**/ {  }
+        """
+        expected = """selector/**/ {  }
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        /* \*/
+        rules
+        /* */
+        """
+        expected = """/* \*/
+rules
+/* */
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        /* foobar \*/
+        rules
+        /* */
+        """
+        expected = """/* \*/
+rules
+/* */
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        /*/*/
+        rules
+        /* */
+        """
+        expected = """/*/*/
+rules
+/* */
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        /*/*//*/
+        rules
+        /* */
+        """
+        expected = """/*/*//*/
+rules
+/* */
+"""
+        got = self.tool._compressCSS(input, 'safe')
+        self.assertEqual(got, expected)
+
+        input = """
+        /* test string protection and full compression */
+        #selector,
+        #another {
+            content: 'foo; bar';
+        }
+        """
+        expected = """/* */
+#selector,#another {content: 'foo; bar';}
+"""
+        got = self.tool._compressCSS(input, 'full')
+        self.assertEqual(got, expected)
+
+
 def test_suite():
     from unittest import TestSuite, makeSuite
     suite = TestSuite()
@@ -1000,6 +1179,7 @@ def test_suite():
     suite.addTest(makeSuite(TestDebugMode))
     suite.addTest(makeSuite(TestResourceObjects))
     suite.addTest(makeSuite(TestSkinAwareness))
+    suite.addTest(makeSuite(TestCSSCompression))
 
     if not PLONE21:
         # We must not test for the defaults in Plone 2.1 because they are
